import * as log from './log'
import * as utils from './utils'

export class EspThermError extends Error {
    constructor(message : any) {
        super(message)
    }
}

export class EspThermData {
    private temperature : number
    private voltage : number
    private timeConsumed : number

    constructor(temperature:number, voltage:number, timeConsumed:number) {
        this.temperature = temperature
        this.voltage = voltage
        this.timeConsumed = timeConsumed
    }

    toString() :string {
        return JSON.stringify(this)
    }

    toJSON() : any {
        return utils.jsonPrepaper(this, [])
    }
}

export class EspThermMetadata {
    private client : string
    private timestamp : Date

    constructor(client : string, timestamp? : Date) {
        this.client = client
        this.timestamp = (timestamp) ? timestamp : new Date()
    }

    toString() :string {
        return JSON.stringify(this)
    }

    toJSON() : any {
        return utils.jsonPrepaper(this, [])
    }
}

export class EspThermMessage {
    private data : EspThermData
    private metadata : EspThermMetadata


    constructor(client:string, temperature:number, voltage:number, timeConsumed:number) {
        this.data = new EspThermData(temperature, voltage, timeConsumed)
        this.metadata = new EspThermMetadata(client)
    }

    toString() :string {
        return JSON.stringify(this)
    }

    toJSON() : any {
        return utils.jsonPrepaper(this, [])
    }
}

export function espThermToJson(message : any) : any {
    let messageStr : string = "" + <string>message
    let parts : string[] = messageStr.split(' ')

    if (parts.length != 4) {
        throw new EspThermError(`too few or too much, ${parts.length} elements in message, it was "${message}"`)
    }

    let clientId = parts[0]
    let temperature = parseFloat(parts[1])
    log.info(`temperature: ${temperature}`)
    if ((typeof(temperature)  != "number") || (temperature == null) || isNaN(temperature)) {
        throw new EspThermError(`temperature element should be a number, message was "${message}"`)
    }
    let voltage = parseFloat(parts[2])
    if ((typeof(voltage) != "number") || (voltage == null) || isNaN(voltage)) {
        throw new EspThermError(`voltage element should be a number, message was "${message}"`)
    }
    let timeConsumed = parseInt(parts[3])
    if ((typeof(timeConsumed) != "number") || (timeConsumed == null) || isNaN(timeConsumed)) {
        throw new EspThermError(`timeConsumed element should be a number, message was "${message}"`)
    }




    let espThermMessage : EspThermMessage = new EspThermMessage(clientId, 
        temperature, voltage, timeConsumed)
    return espThermMessage
}