import * as CallChain from './callchain'
import * as log from './log'
import * as MongoDB from 'mongodb'


export interface MongoItem {
    id : MongoDB.ObjectID
    payload : any
}

export class MongoSave extends CallChain.AAsyncBaseChainItem  {
    private url : string
    private mongoClient : MongoDB.MongoClient
    private dbh : MongoDB.Db | undefined
    private connectPending : boolean

    constructor(url:string) {
        super('MongoSave')
        this.url = url
        this.mongoClient = new MongoDB.MongoClient()
        this.connectPending = false
    }

    protected func(message : any, finished : (message : any) => void) : void {
        if (! this.dbh) {
            log.info("MongoSave: Not database connection yet")
            if (! this.connectPending) {
                this.connectPending = true
                this.mongoClient.connect(this.url)
                    .then((db:MongoDB.Db) => { 
                        log.info("MongoSave: Successfully opened MongoDB connect")
                        this.dbh = db
                    })
                    .catch((err) => {
                        log.error(`MongoSave: Failure when opening MongoDB connect: ${err}`)
                        this.dbh = undefined
                    })
            } else {
                log.info("MongoSave: Connecting to database is pending")
            }
        }
        if (this.dbh) {
            log.info("MongoSave: Database handle is available")
            let coll : MongoDB.Collection = this.dbh.collection("iot")
            coll.insertOne(message)
                .then((res : MongoDB.InsertOneWriteOpResult) => {
                    log.info(`MongoSave: Successfully wrote one item in database: ${res.insertedId}`)
                    let nextValue = <MongoItem>{id: res.insertedId, payload: message}
                    finished(nextValue)
                })
                .catch((err : any) => {
                    log.error(`MongoSave: Failure when trying to write one item in database: ${err}`)
                    log.error("MongoSave: Chain interrupted")
                })
        } else {
            log.error(`MongoSave: No database connection yet, drop message ${message}`)
            log.error("MongoSave: Chain interrupted")
        }
        log.info(`MongoSave: Returning from ${this.label}`)
    }
}
